/*  Copyright 2005 Fabien Coulon

    This file is part of OpenEuclide

    OpenEuclide is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    OpenEuclide is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with OpenEuclide; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "binreloc.h"
#include "mlstring.h"
#include "euclideConf.h"
#include "euclideMain.h"
#include "id.h"
#include <ios>
using namespace std;

#include "itemFactory.cxx"

mlEntry::mlEntry( const string& filename ) {

  ifstream is; is.open( ((string)(gbr_find_data_dir("/usr/share")) + "/openeuclide/" +filename).c_str(), ios_base::in );

  if ( readThisLine( is ) ) sv = new mlEntry( is );
  else {
    sv = NULL;
    cerr << "Euclide : Could not read any entry in the multi-language file " << gbr_find_data_dir("/usr/share") << "/openeuclide/" << filename << " !!\n";
  }
  is.close();
}

mlEntry::mlEntry( ifstream& is ) {

  if ( readThisLine( is ) ) sv = new mlEntry( is );
  else sv = NULL;
}

int  mlEntry::readThisLine( ifstream& is ) {

  while ( is.good() && ( (char)(is.get()) != '@' ) ) ;;
  
  if ( ! is.good() ) { bValid = 0; return 0; }
  
  idx = 0;
  is >> idx;
  if ( !idx ) { cerr << "Euclide : Bad index number found in the multi-language file !!\n"; bValid = 0; return 0; }
  
  int  i=0;
  char tmp[MAX_MLSTRING_LENGTH];
  
  while ( is.good() && ( (char)(is.get()) != '\"' ) ) ;;
  while ( is.good() && ( i<MAX_MLSTRING_LENGTH-1 ) && (( tmp[i++] = (char)(is.get()) ) != '\"' )) ;;
  
  if ( !i ) { cerr << "Euclide : An entry of the multi-language file is empty\n"; }
  if ( i==MAX_MLSTRING_LENGTH-1 ) { cerr << "Euclide : An entry of the multi-language file is too big\n"; }

  tmp[--i] = 0;
  _str = tmp;

  bValid = 1;
  return 1;
}

const string* mlEntry::get( int id ) {

  if ( bValid ) {
    
    if ( idx == id ) return &_str;
    return sv->get( id );
  }
  return NULL;
}


mlEntry* mlString::mlentry = NULL;
string   mlString::mlFileName = "";
string* mlString::str_empty = new string("");

mlString::mlString() { 

  empty();
}

mlString::mlString( int idx ) {

  _str = mlentry->get( idx );
  if (!_str) empty();
}

void mlString::loadString( int idx ) {

  if ( !mlentry ) mlentry = new mlEntry( mlFileName );
  _str = mlentry->get( idx );
  if (!_str) empty();
}

void mlString::empty() {

  _str = str_empty;
}

GtkItemFactoryEntry* mlString::elementsMenu() {

   if ( mlFileName == "english.multilanguage" ) 
      return elements_menu_en;                     
   if ( mlFileName == "french.multilanguage" ) 
      return elements_menu_fr;
   if ( mlFileName == "catala.multilanguage" ) 
      return elements_menu_ca;
   if ( mlFileName == "russian.multilanguage" ) 
      return elements_menu_ru;
   return elements_menu_en;
}

int mlString::nbElementsMenu() {
 
   if ( mlFileName == "english.multilanguage" ) 
     return sizeof (elements_menu_en) / sizeof (elements_menu_en[0]);      
   if ( mlFileName == "french.multilanguage" ) 
     return sizeof (elements_menu_fr) / sizeof (elements_menu_fr[0]);      
   if ( mlFileName == "catala.multilanguage" ) 
     return sizeof (elements_menu_ca) / sizeof (elements_menu_ca[0]);
   if ( mlFileName == "russian.multilanguage" ) 
     return sizeof (elements_menu_ru) / sizeof (elements_menu_ru[0]);
   return sizeof (elements_menu_en) / sizeof (elements_menu_en[0]);      
}

void mlString::select() {

  const char* name = config::getString( "language" );
  if ( !name ) {
    mlFileName = "english.multilanguage";
    config::setString( "language", mlFileName.c_str() );
  } else mlFileName = name;

  mlentry = new mlEntry( mlFileName );
}

void mlString::force( int id ) {

  char *names[4] = { "english.multilanguage", "french.multilanguage", "catala.multilanguage", "russian.multilanguage" };

  mlFileName = names[id];
  config::setString( "language", mlFileName.c_str() );

  delete mlentry;
  mlentry = new mlEntry( mlFileName );
}
